@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf8
 
-""" Simple keyboard usage using custom numeric layout. """
+"""Simple keyboard usage using custom numeric layout."""
 
 # pylint: disable=import-error
 import pygame
@@ -28,14 +28,14 @@ def main(test=False):
 
     # Create keyboard
     model = ['123', '456', '789', '*0#']
-    layout = vkboard.VKeyboardLayout(
-        model,
-        key_size=40,
-        allow_uppercase=False,
-        allow_special_chars=False,
-        allow_space=False)
-    keyboard = vkboard.VKeyboard(screen, on_key_event, layout)
-    keyboard.enable()
+    layout = vkboard.VKeyboardLayout(model,
+                                     key_size=40,
+                                     allow_uppercase=False,
+                                     allow_special_chars=False,
+                                     allow_space=False)
+    keyboard = vkboard.VKeyboard(screen, on_key_event, layout,
+                                 special_char_layout=None,
+                                 joystick_navigation=True)
 
     # Main loop
     while True:
