@@ -33,9 +33,15 @@ from .vrenderers import VKeyboardRenderer
 from .vtextinput import VTextInput, VBackground
 
 
-# Configure logger.
+# Configure LOGGER.
 logging.basicConfig()
-logger = logging.getLogger(__name__)
+LOGGER = logging.getLogger(__name__)
+
+# Joystick controls
+JOYHAT_UP = (0, 1)
+JOYHAT_LEFT = (-1, 0)
+JOYHAT_RIGHT = (1, 0)
+JOYHAT_DOWN = (0, -1)
 
 
 def synchronize_layouts(surface_size, *layouts):
@@ -161,13 +167,13 @@ class VKeyboardLayout(object):
     AZERTY = ['1234567890', 'azertyuiop', 'qsdfghjklm', 'wxcvbn']
 
     # QWERTY Layout.
-    QWERTY = ['1234567890', 'qwertyuiop', 'asdfghjkl', 'wzxcvbnm']
+    QWERTY = ['1234567890', 'qwertyuiop', 'asdfghjkl', 'zxcvbnm']
 
     # Number only layout.
     NUMBER = ['123', '456', '789', '0']
 
     # TODO : Insert special characters layout which include number.
-    SPECIAL = [u'&é"\'(§è!çà)', u'°_-^$¨*ù`%£', u',;:=?.@+<>#', u'[]{}/\\|']
+    SPECIAL = [u'&é"\'(§è!çà)', u'°_-^$¨*ù`%£', u',;:=?.@+<>#', u'€[]{}/\\|']
     """Special characters layout. """
 
     def __init__(self,
@@ -205,6 +211,7 @@ class VKeyboardLayout(object):
         self.sprites = pygame.sprite.LayeredDirty()
         self.key_size = key_size
         self.padding = padding
+        self.selection = None
         self.allow_space = allow_space
         self.allow_uppercase = allow_uppercase
         self.allow_special_chars = allow_special_chars
@@ -264,7 +271,7 @@ class VKeyboardLayout(object):
         self.sprites.add(*special_keys, layer=1)
 
         # Dispatch special keys in the layout
-        while len(special_keys) > 0:
+        while special_keys:
             first = False
             while len(special_keys) > 0 and len(current_row) < max_length:
                 current_row.add_key(special_keys.pop(0), first=first)
@@ -281,10 +288,10 @@ class VKeyboardLayout(object):
         first = True
 
         # Adding left to the special bar.
-        while len(special_keys) > 0:
+        while special_keys:
             special_row.add_key(special_keys.pop(0), first=first)
             first = not first
-        if len(special_row) > 0:
+        if special_row:
             self.rows.append(special_row)
 
     def configure_bound(self, surface_size):
@@ -306,7 +313,7 @@ class VKeyboardLayout(object):
             self.key_size = int(((surface_size[1] / 2)
                                  - (self.padding * (r + 1))) / r)
             height = self.key_size * r + self.padding * (r + 1)
-            logger.warning('Computed layout height outbound target surface,'
+            LOGGER.warning('Computed layout height outbound target surface,'
                            ' reducing key_size to %spx', self.key_size)
         self.set_size((surface_size[0], int(height)), surface_size)
 
@@ -345,7 +352,7 @@ class VKeyboardLayout(object):
             key.set_uppercase(uppercase)
 
     def get_key(self, value):
-        """Retrieves if any key with the given value
+        """Retrieve if any key with the given value
 
         Parameters
         ----------
@@ -363,7 +370,7 @@ class VKeyboardLayout(object):
         return None
 
     def get_key_at(self, position):
-        """Retrieves if any key is located at the given position
+        """Retrieve if any key is located at the given position
 
         Parameters
         ----------
@@ -378,6 +385,52 @@ class VKeyboardLayout(object):
         for sprite in self.sprites.get_sprites_at(position):
             if isinstance(sprite, vkeys.VKey):
                 return sprite
+        return None
+
+    def get_key_closest(self, key, loop_row=True, loop_col=True):
+        """Retrieve the keys closest to the given one. It returns
+        a dictionary with closest keys and their position relative
+        to the given key (diff row, diff column).
+
+        Parameters
+        ----------
+        key:
+            The key to locate.
+        loop_row:
+            Loop to the start/end of the row when right/left overflow.
+        loop_col:
+            Loop to the start/end of the column when bottom/top overflow.
+
+        Returns
+        -------
+        keys_dict:
+            The located keys on the left, top, right, bottom.
+        """
+        center = (0, 0)
+        left = (0, -1)
+        top = (-1, 0)
+        right = (0, 1)
+        bottom = (1, 0)
+        keys_dict = {center: key,
+                     left: None, top: None, right: None, bottom: None}
+
+        for row_index, r in enumerate(self.rows):
+            for key_index, k in enumerate(r.keys):
+                if key == k:
+                    if key_index - 1 >= 0 or loop_row:
+                        keys_dict[left] = r.keys[(key_index - 1) % len(r.keys)]
+                    if key_index + 1 < len(r.keys) or loop_row:
+                        keys_dict[right] = r.keys[(key_index + 1) % len(r.keys)]
+
+                    if row_index - 1 >= 0 or loop_col:
+                        prev_row = self.rows[(row_index - 1) % len(self.rows)]
+                        keys_dict[top] = prev_row\
+                            .keys[min(key_index, len(prev_row) - 1)]
+                    if row_index + 1 < len(self.rows) or loop_col:
+                        next_row = self.rows[(row_index + 1) % len(self.rows)]
+                        keys_dict[bottom] = next_row\
+                            .keys[min(key_index, len(next_row) - 1)]
+        return keys_dict
 
 
 class VKeyboard(object):
@@ -393,9 +446,10 @@ class VKeyboard(object):
                  surface,
                  text_consumer,
                  layout,
-                 special_char_layout=VKeyboardLayout(VKeyboardLayout.SPECIAL),
+                 show_text=False,
+                 joystick_navigation=False,
                  renderer=VKeyboardRenderer.DEFAULT,
-                 show_text_input=False):
+                 special_char_layout=VKeyboardLayout(VKeyboardLayout.SPECIAL)):
         """ Default constructor.
 
         Parameters
@@ -406,20 +460,25 @@ class VKeyboard(object):
             Consumer that process text for each update.
         layout:
             Layout this keyboard will use.
-        special_char_layout:
-            Alternative layout to use, using VKeyboardLayout.SPECIAL
-            if not specified.
+        show_text:
+            Display the current text in a text box.
+        joystick_navigation:
+            Handle joystick events and arrow keys.
         renderer:
             Keyboard renderer instance, using VKeyboardRenderer.DEFAULT
             if not specified.
+        special_char_layout:
+            Alternative layout to use, using VKeyboardLayout.SPECIAL
+            if not specified.
         """
         self.surface = surface
         self.text_consumer = text_consumer
         self.renderer = renderer
-        self.state = 0
+        self.state = 1  # Enabled by default
         self.last_pressed = None
         self.uppercase = False
         self.special_char = False
+        self.joystick_navigation = joystick_navigation
 
         self.layout = layout
         self.original_layout = layout
@@ -427,27 +486,40 @@ class VKeyboard(object):
         self.original_layout.configure_renderer(self.renderer)
 
         self.special_char_layout = special_char_layout
-        self.special_char_layout.configure_special_keys(self)
-        self.special_char_layout.configure_renderer(self.renderer)
+        if self.special_char_layout:
+            self.special_char_layout.configure_special_keys(self)
+            self.special_char_layout.configure_renderer(self.renderer)
+            synchronize_layouts(self.surface.get_size(),
+                                self.original_layout,
+                                self.special_char_layout)
+        else:
+            synchronize_layouts(self.surface.get_size(), self.original_layout)
 
-        synchronize_layouts(self.surface.get_size(), self.original_layout,
-                            self.special_char_layout)
-
-        self.background = VBackground(
-            self.surface.get_rect().size,
-            self.renderer)
+        self.eraser = None
+        self.background = VBackground(self.surface.get_rect().size,
+                                      self.renderer)
         self.background.set_rect(*self.layout.position + self.layout.size)
         self.original_layout.sprites.add(self.background, layer=0)
-        self.special_char_layout.sprites.add(self.background, layer=0)
+        if self.special_char_layout:
+            self.special_char_layout.sprites.add(self.background, layer=0)
 
-        self.input = VTextInput((
-            self.original_layout.position[0],
-            self.original_layout.position[1] - self.original_layout.key_size),
-            (self.original_layout.size[0], self.original_layout.key_size))
+        self.show_text = show_text
+        self.input = VTextInput((self.original_layout.position[0],
+                                 self.original_layout.position[1]
+                                 - self.original_layout.key_size),
+                                (self.original_layout.size[0],
+                                 self.original_layout.key_size),
+                                renderer=self.renderer)
 
-        if show_text_input:
+        if self.show_text:
             self.input.enable()
 
+        if self.joystick_navigation:
+            if not pygame.joystick.get_init():
+                pygame.joystick.init()
+            for i in range(pygame.joystick.get_count()):
+                pygame.joystick.Joystick(i).init()
+
     def set_layout(self, layout):
         """Sets the layout this keyboard work with.
         Keyboard is invalidate by this action and redraw itself.
@@ -461,13 +533,43 @@ class VKeyboard(object):
         self.layout = layout
         self.layout.show()
 
+    def set_text(self, text):
+        """Set the current text in the internal buffer.
+
+        Parameters
+        ----------
+        text:
+            Text to be set.
+        """
+        self.input.set_text(text)
+
+    def get_text(self):
+        """Return the current text of the internal buffer."""
+        return self.input.text
+
     def enable(self):
-        """Sets this keyboard as active. """
+        """Set this keyboard as active."""
         self.state = 1
+        self.layout.show()
+        if self.show_text:
+            self.input.enable()
+
+    def is_enabled(self):
+        """Return True if this keyboard is active."""
+        return self.state == 1
 
     def disable(self):
-        """Sets this keyboard as non active. """
+        """Set this keyboard as non active."""
         self.state = 0
+        self.layout.hide()
+        self.input.disable()
+
+    def get_rect(self):
+        """Return keyboard rect."""
+        rect = self.background.rect
+        if self.show_text:
+            rect = rect.union(self.input.get_rect())
+        return rect
 
     def draw(self, surface=None, force=False):
         """Draw the virtual keyboard.
@@ -488,12 +590,20 @@ class VKeyboard(object):
         rects:
             List of updated area.
         """
-        rects = []
-        if self.state > 0:
-            rects += self.layout.sprites.draw(surface or self.surface)
-            rects += self.input.draw(surface or self.surface, force)
-            if force:
-                self.layout.sprites.repaint_rect(self.background.rect)
+        # Setup the surface used to hide/clear the keyboard
+        if surface and surface != self.eraser:
+            self.eraser = surface
+            self.original_layout.sprites.clear(surface, self.eraser.copy())
+            self.original_layout.sprites.set_clip(self.background.rect)
+            if self.special_char_layout:
+                self.special_char_layout.sprites.clear(surface,
+                                                       self.eraser.copy())
+                self.special_char_layout.sprites.set_clip(self.background.rect)
+
+        rects = self.layout.sprites.draw(surface or self.surface)
+        rects += self.input.draw(surface or self.surface, force)
+        if force:
+            self.layout.sprites.repaint_rect(self.background.rect)
         return rects
 
     def update(self, events):
@@ -509,14 +619,80 @@ class VKeyboard(object):
             self.input.update(events)
 
             for event in events:
-                if event.type == pygame.MOUSEBUTTONDOWN:
+                if event.type == pygame.MOUSEBUTTONDOWN\
+                                and event.button in (1, 2, 3):
+                    # Don't consider the mouse wheel (button 4 & 5)
                     key = self.layout.get_key_at(event.pos)
                     if key:
                         self.on_key_down(key)
+                        self.on_select(0, 0, key)
                 elif event.type == pygame.KEYDOWN:
                     key = self.layout.get_key(event.unicode or event.key)
                     if key:
                         self.on_key_down(key)
+                    if event.key == pygame.K_LEFT and not self.input.selected:
+                        self.on_select(0, -1)
+                    elif event.key == pygame.K_UP:
+                        self.on_select(-1, 0)
+                    elif event.key == pygame.K_RIGHT and\
+                            not self.input.selected:
+                        self.on_select(0, 1)
+                    elif event.key == pygame.K_DOWN:
+                        self.on_select(1, 0)
+                    elif event.key == pygame.K_RETURN\
+                            and self.layout.selection:
+                        self.on_key_down(self.layout.selection)
+                elif event.type == pygame.JOYHATMOTION:
+                    if event.value == JOYHAT_LEFT and\
+                            not self.input.selected:
+                        self.on_select(0, -1)
+                    elif event.value == JOYHAT_UP:
+                        self.on_select(-1, 0)
+                    elif event.value == JOYHAT_RIGHT and\
+                            not self.input.selected:
+                        self.on_select(0, 1)
+                    elif event.value == JOYHAT_DOWN:
+                        self.on_select(1, 0)
+                    elif event.type == pygame.JOYBUTTONDOWN\
+                            and event.button == 0 and self.layout.selection:
+                        # Select button pressed
+                        self.on_key_down(self.layout.selection)
+
+    def on_select(self, increment_row, increment_col, key=None):
+        """"Change the currently selected key.
+
+        Parameters
+        ----------
+        increment_row:
+            Of how many rows to move (-1 to go up, 1 to go down).
+        increment_col:
+            Of how many columns to move (-1 to go left, 1 to go right).
+        key:
+            The key to start from, by default the currently selected one.
+        """
+        if self.joystick_navigation:
+            if not self.layout.selection:
+                if key:
+                    self.layout.selection = key
+                elif increment_row > 0:
+                    self.layout.selection = self.layout.rows[0].keys[0]
+                else:
+                    self.layout.selection = self.layout.rows[-1].keys[0]
+                self.layout.selection.set_selected(1)
+                self.input.set_selected(0)
+                return
+
+            closest = self.layout.get_key_closest(
+                                        key or self.layout.selection,
+                                        loop_col=not self.input.is_enabled())
+
+            self.layout.selection.set_selected(0)
+            self.layout.selection = None
+            if closest[(increment_row, increment_col)]:
+                self.layout.selection = closest[(increment_row, increment_col)]
+                self.layout.selection.set_selected(1)
+            else:
+                self.input.set_selected(1)
 
     def on_event(self, event):
         """Deprecated method, only for backward compatibility."""
@@ -527,15 +703,17 @@ class VKeyboard(object):
         """Uppercase key press handler."""
         self.uppercase = not self.uppercase
         self.original_layout.set_uppercase(self.uppercase)
-        self.special_char_layout.set_uppercase(self.uppercase)
+        if self.special_char_layout:
+            self.special_char_layout.set_uppercase(self.uppercase)
 
     def on_special_char(self):
         """Special char key press handler."""
-        self.special_char = not self.special_char
-        if self.special_char:
-            self.set_layout(self.special_char_layout)
-        else:
-            self.set_layout(self.original_layout)
+        if self.special_char_layout:
+            self.special_char = not self.special_char
+            if self.special_char:
+                self.set_layout(self.special_char_layout)
+            else:
+                self.set_layout(self.original_layout)
 
     def on_key_down(self, key):
         """Process key down event by pressing the given key.
